@@ -9,13 +9,15 @@ use Clustericious::Config 1.03;
 use Term::ANSIColor ();
 use Sys::Hostname qw( hostname );
 use YAML::XS qw( Dump );
-use JSON::MaybeXS qw( encode_json );
 use File::Basename qw( basename );
 use AE;
 use Clustericious::Admin::RemoteHandler;
+use Clustericious::Admin::Dump qw( perl_dump );
+use File::chdir;
+use Path::Class ();
 
 # ABSTRACT: Parallel SSH client
-our $VERSION = '1.02'; # VERSION
+our $VERSION = '1.03'; # VERSION
 
 
 sub main
@@ -62,6 +64,7 @@ sub new
     'fat'      => \$self->{fat},
     'max=s'    => \$self->{max},
     'file=s'   => $self->{files},
+    'dir=s'    => \$self->{dir},
     'help|h'   => sub { pod2usage({ -verbose => 2}) },
     'version'  => sub {
       say STDERR 'App::clad version ', ($App::clad::VERSION // 'dev');
@@ -130,6 +133,12 @@ sub new
     $ok = 0;
   }
   
+  if(defined $self->dir && ! -d $self->dir)
+  {
+    say STDERR "unable to find @{[ $self->dir ]}";
+    $ok = 0;
+  }
+  
   exit 2 unless $ok;
   
   $self;
@@ -146,6 +155,7 @@ sub verbose        { shift->{verbose}       }
 sub serial         { shift->{serial}        }
 sub max            { shift->{max}           }
 sub files          { @{ shift->{files} }    }
+sub dir            { shift->{dir}           }
 sub script         { @{ shift->{script} // [] } }
 sub ssh_command    { shift->config->ssh_command(    default => 'ssh' ) }
 sub ssh_options    { shift->config->ssh_options(    default => [ -o => 'StrictHostKeyChecking=no', 
@@ -251,7 +261,7 @@ sub payload
       $h{content} = do { local $/; <$fh> };
       close $fh;
       $h{name} = basename $filename;
-      $h{mode} = (stat "/etc/passwd")[2] & 0777;
+      $h{mode} = sprintf "%o", (stat $filename)[2] & 0777;
       push @{ $payload->{files} }, \%h;
     }
   }
@@ -269,13 +279,44 @@ sub payload
     };
   }
   
+  if($self->dir)
+  {
+    $payload->{require} = '1.02';
+    
+    $CWD = $self->dir;
+    
+    my $recurse;
+    $recurse = sub {
+      my($dir) = @_;
+      foreach my $child ($dir->children(no_hidden => 1))
+      {
+        my $key = $child->relative->stringify;
+        if($child->is_dir)
+        {
+          $payload->{dir}->{$key} = {
+            is_dir => 1,
+          };
+          $recurse->($child);
+        }
+        else
+        {
+          $payload->{dir}->{$key} = {
+            content => scalar $child->slurp(iomode => '<:bytes'),
+          };
+        }
+        $payload->{dir}->{$key}->{mode} = sprintf '%o', $child->stat->mode & 0777;
+      }
+    };
+    
+    $recurse->(Path::Class::Dir->new);
+  }
+  
   if($self->fat)
   {
     # Perl on the remote end may not have YAML
-    # (actually it may not have JSON::PP either
-    # but at least that is part of the core as
-    # of 5.14).
-    $payload = encode_json($payload);
+    # so we dump as Perl data structure
+    # instead.
+    $payload = perl_dump($payload);
     require Clustericious::Admin::Server;
     open my $fh, '<', $INC{'Clustericious/Admin/Server.pm'};
     my $code = do { local $/; <$fh> };
@@ -373,7 +414,7 @@ App::clad - Parallel SSH client
 
 =head1 VERSION
 
-version 1.02
+version 1.03
 
 =head1 SYNOPSIS
 
