@@ -8,7 +8,7 @@ use File::Spec;
 use File::Path qw( mkpath );
 
 # ABSTRACT: Parallel SSH client server side code
-our $VERSION = '1.02'; # VERSION
+our $VERSION = '1.03'; # VERSION
 
 
 # This is the implementation of the clad server.
@@ -25,9 +25,9 @@ our $VERSION = '1.02'; # VERSION
 # payload is preferred because it is easier to read
 # when things go wrong.  If App::clad is NOT installed
 # on the remote end, then you can take this pm file,
-# append the payload as JSON after the __DATA__ section
-# below and send the server and payload and feed it
-# into perl on the remote end.
+# append the payload as Perl Dump after the __DATA__
+# section below and send the server and payload and
+# feed it into perl on the remote end.
 
 sub _decode
 {
@@ -52,21 +52,27 @@ sub _decode
     }
     print STDERR YAML::XS::Dump($payload) if $payload->{verbose};
   }
-  else
+  elsif($raw =~ /^#perl/)
   {
-    eval {
-      require JSON::PP;
-      $payload = JSON::PP::decode_json($raw);
-    };
-    if(my $json_error = $@)
+    $payload = eval $raw;
+    if(my $perl_error = $@)
     {
-      print STDERR "Clad Server: side YAML/JSON Error:\n";
-      print STDERR $json_error, "\n";
+      print STDERR "Clad Server: side Perl Error:\n";
+      print STDERR $perl_error, "\n";
       print STDERR "payload:\n";
       print STDERR $raw, "\n";
       return;
     }
-    print JSON::PP::encode_json($payload) if $payload->{verbose};
+    eval {
+      require Data::Dumper;
+      print Dumper($payload) if $payload->{verbose};
+    };
+  }
+  else
+  {
+    print STDERR "Clad Server: unable to detect encoding.\n";
+    print STDERR "payload:\n";
+    print STDERR $raw;
   }
   
   $payload;
@@ -143,7 +149,7 @@ sub _server
     {
       my $path = File::Spec->catfile( tempdir( CLEANUP => 1 ), $file->{name} );
       open my $fh, '>', $path;
-      chmod oct($file->{mode}), $fh if defined $file->{mode};
+      chmod oct($file->{mode}), $path if defined $file->{mode};
       binmode $fh;
       print $fh $file->{content};
       close $fh;
@@ -197,7 +203,7 @@ sub _server
   return $? >> 8;
 }
 
-__PACKAGE__->_server(*DATA) unless caller;
+exit __PACKAGE__->_server(*DATA) unless caller;
 
 1;
 
@@ -211,7 +217,7 @@ Clustericious::Admin::Server - Parallel SSH client server side code
 
 =head1 VERSION
 
-version 1.02
+version 1.03
 
 =head1 SYNOPSIS
 
